/*

  PONGDuino : a Pong game for Arduino and 16x2 LCD display

  Copyright (c) 2013, Xavier Miller (xavier.miller@cauwe.org)
  http://www.xaviermiller.be

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  - Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

  - Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

*/

#include "inttypes.h"

#include "pong.h"

#include "LiquidCrystal.h"

#include "lcdpong.h"

LCDPong::LCDPong()
: LiquidCrystal(12, 11, 5, 4, 3, 2)
{
    begin(16, 2);
    m_pong = new Pong(this, 12*5, 2*8);
}

LCDPong::~LCDPong()
{
    delete m_pong;
}

void LCDPong::timerEvent()
{
    if (m_running)
        m_pong->TimerTick();
}


void LCDPong::Pause()
{
    m_running = false;
}

void LCDPong::Continue()
{
  m_running = true;
}

void LCDPong::DisplayScore(int score1, int score2)
{
    if (score1 > 9)
    {
        setCursor(0, 0);
        write('0' + (score1 / 10));
        setCursor(1, 0);
        write('0' + (score1 % 10));
    }
    else
    {
        setCursor(0, 0);
        write('0' + score1);
    }

    if (score2 > 9)
    {
        setCursor(14, 0);
        write('0' + (score2 / 10));
        setCursor(15, 0);
        write('0' + (score2 % 10));
    }
    else
    {
        setCursor(15, 0);
        write('0' + score2);
    }
}

void LCDPong::PauseScreen(int score1, int score2, const unsigned char *message)
{
    clear();

    DisplayScore(score1, score2);
    setCursor(0, 1);
    print((const char*)message);
}

void LCDPong::GameScreen(int score1, int score2, int ball_x, int ball_y, int ball_size, int /*paddle1_x*/, int paddle1_y, int /*paddle2_x*/, int paddle2_y, int /*paddle_width*/, int paddle_height)
{
    clear();

    // Display score
    DisplayScore(score1, score2);

    unsigned char buf1[16];
    unsigned char buf2[16];
    unsigned char *buf;

    // compute and display ball
    int ball_col = 2 + ball_x / 5;
    ball_x %= 5;
    int ball_mask = 24 >> ball_x;
    buf = buf1;
    for (int h = 0; h < 16; h++)
        buf[h] = ((ball_y <= h) && (h < ball_y + ball_size)) ? ball_mask : 0;
    buf = buf2;
    if (ball_x == 4)
        for (int h = 0; h < 16; h++)
            buf[h] = ((ball_y <= h) && (h < ball_y + ball_size)) ? 16 : 0;
    else
        memset(buf, 0, 16);

    if (ball_col == 13)
    {
        ball_col--;
        memcpy(buf2, buf1, 16);
        memset(buf1, 0, 16);
    }
    else
    {
        // not on the left paddle colum or right
        if (ball_col != 2)
        {
            buf = buf1;
            createChar(0, buf);
            createChar(1, buf+8);
            setCursor(ball_col, 0);
            write((unsigned char)0);
            setCursor(ball_col, 1);
            write((unsigned char)1);
            memset(buf, 0, 16);
        }
        // not on the right paddle column
        if (ball_col < 12)
        {
            buf = buf2;
            createChar(2, buf);
            createChar(3, buf+8);
            setCursor(ball_col+1, 0);
            write((unsigned char)2);
            setCursor(ball_col+1, 1);
            write((unsigned char)3);
            memset(buf, 0, 16);
        }
    }
    // Display Paddles
    // paddle 1
    // we assume paddle1_x = 0
    buf = buf1;
    for (int h = 0; h < 16; h++)
        buf[h] |= ((paddle1_y <= h) && (h < paddle1_y + paddle_height)) ? 24 : 0;
    createChar(4, buf);
    createChar(5, buf+8);
    setCursor(2, 0);
    write((unsigned char)4);
    setCursor(2, 1);
    write((unsigned char)5);

    // paddle 1
    // we assume paddle2_y = width-2
    // top
    buf = buf2;
    for (int h = 0; h < 16; h++)
        buf[h] |= ((paddle2_y <= h) && (h < paddle2_y + paddle_height)) ? 3 : 0;
    createChar(6, buf);
    createChar(7, buf+8);
    setCursor(13, 0);
    write((unsigned char)6);
    setCursor(13, 1);
    write((unsigned char)7);
}
