/*

  PONGDuino : a Pong game for Arduino and 16x2 LCD display

  Copyright (c) 2013, Xavier Miller (xavier.miller@cauwe.org)
  http://www.xaviermiller.be

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  - Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

  - Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

*/

#include "pong.h"
#include "LiquidCrystal.h"
#include "lcdpong.h"

// LCD 16x2 setup tutorial: http://arduino.cc/en/uploads/Tutorial/LCD_bb.png

#define KEY_COUNT 5

#define PAUSE_BUTTON  10
#define LEFT_1 6
#define RIGHT_1 7
#define LEFT_2 8
#define RIGHT_2 9

LCDPong *lcdpong;

byte buttons[KEY_COUNT] = { PAUSE_BUTTON, LEFT_1, RIGHT_1, LEFT_2, RIGHT_2 };
int buttonState[KEY_COUNT] = { LOW, LOW, LOW, LOW, LOW };
long lastDebounceTime[KEY_COUNT] = { 0, 0, 0, 0, 0 };
byte keyWasDown[KEY_COUNT] = { 1, 1, 1, 1, 1 };

long lastTimer = 0;

#define DEBOUNCE_DELAY 50

void setup()
{
  for (int key = 0; key < KEY_COUNT; key++)
    pinMode(buttons[key], INPUT);
  
  lcdpong = new LCDPong();
}

void loop()
{
  // read keys
  for (byte key = 0; key < KEY_COUNT; key++)
  {
    int reading = digitalRead(buttons[key]);
    if ((millis() - lastDebounceTime[key]) > DEBOUNCE_DELAY)
    {
      if (reading != buttonState[key])
      {
        Serial.print("State changed to ");
        Serial.println(reading);
        buttonState[key] = reading;
        lastDebounceTime[key] = millis();
      }
    }
    if (buttonState[key] == LOW)
      keyWasDown[key] = 1;
  }
  
  // Pause Key
  if ((buttonState[0] == HIGH) && keyWasDown[0])
  {
    keyWasDown[0] = 0;
    lcdpong->keyPressPause();
  }

  // Left 1
  if ((buttonState[1] == HIGH) && keyWasDown[1])
  {
    keyWasDown[1] = 0;
    lcdpong->keyPressUp1();
  }
  // Right 1
  if ((buttonState[2] == HIGH) && keyWasDown[2])
  {
    keyWasDown[2] = 0;
    lcdpong->keyPressDown1();
  }

  // Left 2
  if ((buttonState[3] == HIGH) && keyWasDown[3])
  {
    keyWasDown[3] = 0;
    lcdpong->keyPressUp2();
  }
  // Right 2
  if ((buttonState[4] == HIGH) && keyWasDown[4])
  {
    keyWasDown[4] = 0;
    lcdpong->keyPressDown2();
  }

  if (lcdpong->isRunning())
  {
    if ((millis() - lastTimer) > 100)
    {  
      lcdpong->timerEvent();
      lastTimer = millis();
    }
  }
}
